# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.0.0-beta.1

import enum

import winrt.system
from winrt import _winrt_windows_devices_spi_provider

__all__ = [
    "ProviderSpiMode",
    "ProviderSpiSharingMode",
    "ProviderSpiConnectionSettings",
    "ISpiControllerProvider",
    "ISpiDeviceProvider",
    "ISpiProvider",
]

class ProviderSpiMode(enum.IntEnum):
    MODE0 = 0
    MODE1 = 1
    MODE2 = 2
    MODE3 = 3

class ProviderSpiSharingMode(enum.IntEnum):
    EXCLUSIVE = 0
    SHARED = 1

ProviderSpiConnectionSettings = _winrt_windows_devices_spi_provider.ProviderSpiConnectionSettings
ISpiControllerProvider = _winrt_windows_devices_spi_provider.ISpiControllerProvider
ISpiDeviceProvider = _winrt_windows_devices_spi_provider.ISpiDeviceProvider
ISpiProvider = _winrt_windows_devices_spi_provider.ISpiProvider
